/* Functions to be filled by the user (never overwritten by buildsupport tool) */
#include <stdio.h>
#include <stdint.h>

#include "ground_network.h"

#define HEADER_SIZE 6

unsigned int lookupTable[256]; /* Lookup table for the optimized CRC algorithm */

uint16_t computeCRC(uint8_t * data, size_t length);

void init_ground_network()
{
    printf("[GROUND] Startup \n");
    
    /* Initialize the lookupTable for the CRC algorithm */
    int i;
    for ( i=0; i<256; ++i )
    {
      unsigned int tmp=0;
      if ((i & 1) != 0) tmp=tmp ^ 0x1021;
      if ((i & 2) != 0) tmp=tmp ^ 0x2042;
      if ((i & 4) != 0) tmp=tmp ^ 0x4084;
      if ((i & 8) != 0) tmp=tmp ^ 0x8108;
      if ((i & 16) != 0) tmp=tmp ^ 0x1231;
      if ((i & 32) != 0) tmp=tmp ^ 0x2462;
      if ((i & 64) != 0) tmp=tmp ^ 0x48C4;
      if ((i & 128) != 0) tmp=tmp ^ 0x9188;
      lookupTable[i] = tmp;
    }
}

void ground_network_receiveTMfromSC(void *IN_tm, size_t size_IN_tm)
{
    /*
    * Write your code here! 
    * Note: the parameters are encoded following the rules you specified
    * in the interface view (Native, uPER, or ACN).
    */
    printf("[GROUND] Received telemetry of length %d: \n", size_IN_tm);
    printf("[GROUND] Checking if CRC is correct...");
    int i;
    uint8_t * tm = (uint8_t*) IN_tm;
   
    /* If crc was added correctly and no transmission errors occurred
     * then the CRC of the whole packet (including the CRC bytes) should
     * yield zero */
    uint16_t crc = computeCRC( tm, size_IN_tm );

    /* Overwrite the original CRC fields with hopefully zero,
     * so it is easy to check if the CRC check failed */
    tm[size_IN_tm-2] = (0xFF00 & crc) >> 8;
    tm[size_IN_tm-1] =  0x00FF & crc;
    
    if(crc == 0)
      printf("yes: crc = %d\n", crc);
    else
      printf("no: crc = %d\n", crc);
    
    /* Forward the packet to the GUI */
    vm_async_ground_network_sendTMtoGround( IN_tm, size_IN_tm );
}

void ground_network_receiveTCfromGround(void *IN_tc, size_t size_IN_tc)
{
    /*
    * Write your code here! 
    * Note: the parameters are encoded following the rules you specified
    * in the interface view (Native, uPER, or ACN).
    */
    printf("[GROUND] Received telecommand of length %d: \n", size_IN_tc);
    printf("[GROUND] Adding CRC...");
    int i;
    uint8_t * tc = (uint8_t*) IN_tc;
    printf("[DRIVER] Overwriting size fields...");
    unsigned payload_size = size_IN_tc - HEADER_SIZE -1;
    /* Add size of the TM manually MSB first */
    tc[4] = (0xFF00 & payload_size) >> 8;
    tc[5] =  0x00FF & payload_size;
    printf("done\n");
    
    /* Add Checksum of the TC manually to the last 2 bytes of the stream MSB first */
    uint16_t crc = computeCRC( tc, size_IN_tc-2 );
    tc[size_IN_tc-2] = (0xFF00 & crc) >> 8;
    tc[size_IN_tc-1] =  0x00FF & crc;
    
    printf("done\n");
    
    /* Forward packet to spacecraft network driver */
    vm_async_ground_network_sendTCtoSC( IN_tc, size_IN_tc );
}

/**
 * Optimized CRC-computation copied from ECSS-E70-41A
 */
uint16_t computeCRC(uint8_t * data, size_t length)
{
  uint16_t crc = 0xFFFF; /* start with all ones */
  /* Compute the crc incrementally */
  int i;
  for (i=0; i<length; ++i)
  {
      crc = (((crc << 8) & 0xFF00)^lookupTable[ (((crc >> 8)^data[i]) & 0x00FF) ] );
  }
  
  return crc;
}
