/* Functions to be filled by the user (never overwritten by buildsupport tool) */

#include <stdio.h>
#include "tc_receiver1.h"

/* Function static data is declared in this file : */
#include "Context-tc-receiver1.h"

void tc_receiver1_startup()
{
    printf("[APID %llu] receiver 1 startup\n", tc_receiver1_ctxt.apid);
}

asn1SccT_uint14 getSequenceCount(const asn1SccT_tm_applicationData * appData)
{
  /* Reduce number of global variables and keep the counters here */
  static asn1SccT_uint14 count1_1 = 0;
  static asn1SccT_uint14 count1_2 = 0;
  static asn1SccT_uint14 count1_3 = 0;
  static asn1SccT_uint14 count1_4 = 0;
  static asn1SccT_uint14 count1_7 = 0;
  static asn1SccT_uint14 count1_8 = 0;

  asn1SccT_uint14 result = 0;
  switch(appData->kind)
  {
    case  tm_1_1_PRESENT:
      result = count1_1;
      count1_1 = (count1_1+1) % (1<<14);
      break;
    case  tm_1_2_PRESENT:
      result = count1_2;
      count1_2 = (count1_2+1) % (1<<14);
      break;
    case  tm_1_3_PRESENT:
      result = count1_3;
      count1_3 = (count1_3+1) % (1<<14);
      break;
    case  tm_1_4_PRESENT:
      result = count1_4;
      count1_4 = (count1_4+1) % (1<<14);
      break;
    case  tm_1_7_PRESENT:
      result = count1_7;
      count1_7 = (count1_7+1) % (1<<14);
      break;
    case  tm_1_8_PRESENT:
      result = count1_8;
      count1_8 = (count1_8+1) % (1<<14);
      break;
    default:
      result = 0;
  }
  
  return result;
  
}

void sendTelecommand(const asn1SccT_tm_applicationData * data)
{
    asn1SccT_telemetry tm;
    asn1SccT_telemetry_Initialize(&tm);
    tm.packetHeader.packetID.apid = tc_receiver1_ctxt.apid;
    tm.packetHeader.packetSequenceControl.sequenceControlFlags = asn1SccstandAlonePacket;
    tm.packetHeader.packetSequenceControl.sequenceCount = getSequenceCount(data);
    tm.packetHeader.packetLength = 0;
    tm.applicationData = *data;
    
    tc_receiver1_RI_send1_TM( &tm );    
}

void process7_1(const asn1SccT_telecommand * tc )
{
    
    const asn1SccT_tc_7_1 * data = &tc->applicationData.u.tc_7_1;
    
    // Send status report that execution started if requested
    if(tc->dataFieldHeader.ackExecutionStart)
    {
      asn1SccT_tm_applicationData tmData;
      asn1SccT_tm_applicationData_Initialize( &tmData);
      tmData.kind = tm_1_3_PRESENT;
      tmData.u.tm_1_3.tc_packetID = tc->packetHeader.packetID;
      tmData.u.tm_1_3.tc_packetSequenceControl = tc->packetHeader.packetSequenceControl;
      
      sendTelecommand( &tmData );
    }
    printf("[APID %llu] Received telecommand 7-1 with following parameters:\n",
	   tc_receiver1_ctxt.apid);
    printf("[APID %llu] ", tc_receiver1_ctxt.apid); printf("param1 = %llu \n", data->param1);
    printf("[APID %llu] ", tc_receiver1_ctxt.apid); printf("param2 = %llu \n", data->param2);
    printf("[APID %llu] ", tc_receiver1_ctxt.apid); printf("param3 = %llu \n", data->param3);
    printf("[APID %llu] ", tc_receiver1_ctxt.apid); printf("param4 = %llu \n", data->param4);
    printf("[APID %llu] ", tc_receiver1_ctxt.apid); printf("param5 = %llu \n", data->param5);
    
    // Send status report, that execution finished if requested
    if(tc->dataFieldHeader.ackExecutionCompletion)
    {
      asn1SccT_tm_applicationData tmData;
      asn1SccT_tm_applicationData_Initialize( &tmData);
      tmData.kind = tm_1_7_PRESENT;
      tmData.u.tm_1_7.tc_packetID = tc->packetHeader.packetID;
      tmData.u.tm_1_7.tc_packetSequenceControl = tc->packetHeader.packetSequenceControl;

      sendTelecommand( &tmData );
    }
}

void tc_receiver1_PI_receiver1_TC(const asn1SccT_telecommand * tc)
{
    // Check if the pus service is correct (7-1 as an example)    
    
    asn1SccT_tm_applicationData tmData;
    asn1SccT_tm_applicationData_Initialize( &tmData);
    switch (tc->applicationData.kind)
    {
      case tc_7_1_PRESENT: 
	  // Send successfull acceptance acknowledgement
	  if( tc->dataFieldHeader.ackAcceptance )
	  {
	    tmData.kind = tm_1_1_PRESENT;
	    tmData.u.tm_1_1.tc_packetID = tc->packetHeader.packetID;
	    tmData.u.tm_1_1.tc_packetSequenceControl = tc->packetHeader.packetSequenceControl;
	    sendTelecommand( &tmData );	    
	  }
	  process7_1(tc);
	  break;
      default:
	printf("[APID %llu] Received unknown tc\n", tc_receiver1_ctxt.apid);
	// Send failed acceptance acknowledgement
	if( tc->dataFieldHeader.ackAcceptance )
	{
	  tmData.kind = tm_1_2_PRESENT;
	  tmData.u.tm_1_2.tc_packetID = tc->packetHeader.packetID;
	  tmData.u.tm_1_2.tc_packetSequenceControl = tc->packetHeader.packetSequenceControl;
	  tmData.u.tm_1_2.error_code = asn1SccillegalAPID;
	  sendTelecommand( &tmData );
	}
    }
}

